/**
 * @ButtonPanel.java     18.07.2010
 * build: 18.07.2010
 * Copyright 2010 yourwebs.de. All rights reserved
 * @author J&ouml;rg Czeschla
 *
 * This file is a part of Schiebepuzle
 * Schiebepuzle is free software; you can redistribute it and/or modify it under the terms of the
 * GNU General Public License as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.

 * Spiele is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License along with Spiele;
 * if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA. 
 */
package de.yourwebs.schiebepuzzle;

import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.JToggleButton;

/**
 * defines a Panel with several buttons and its actions
 * 
 */
public class ButtonPanel extends JPanel implements ActionListener, KeyListener {

	private static final long serialVersionUID = 1L;
	
	private PuzzleModel puzzleModel;
	private SchiebePuzzleGUI gui;
	
	// GUI-Components
	private JButton newButt, sortButt, closeButt, helpButt, openButt;
	private JTextField field;
	private JPanel mainPanel;

	// ToggleButtons with its text
	private JToggleButton modeButt, displayButt;
	private String schiebeModusTxt = "Schieben";
	private String springModusTxt = "Springen";
	private String imgDisplayTxt = "Bild-Puzzle";
	private String numbersDisplayTxt = "Zahlen-Puzzle";
	
	public ButtonPanel(SchiebePuzzleGUI gui){
		
		// fetch the  main-window's gui components
		this.gui = gui;
		mainPanel = gui.mainPanel;
		this.setLayout(new BoxLayout(this, BoxLayout.LINE_AXIS));
	}
	
	protected void initButtons() {
		puzzleModel = gui.getPuzzleModel();
		newButt = new JButton();
		newButt.setPreferredSize(new Dimension(22, 22));
		newButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/reload.png")));
		newButt.setToolTipText("Neu mischen");
		newButt.addActionListener(this);

		openButt = new JButton();
		openButt.setPreferredSize(new Dimension(22, 22));
		openButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/fileopen.png")));
		openButt.setToolTipText("Bild laden");
		openButt.addActionListener(this);

		sortButt = new JButton();
		sortButt.setPreferredSize(new Dimension(22, 22));
		sortButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/sort.png")));
		sortButt.setToolTipText("Sortieren");
		sortButt.addActionListener(this);

		closeButt = new JButton();
		closeButt.setPreferredSize(new Dimension(22, 22));
		closeButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/exit.png")));
		closeButt.setToolTipText("Beenden");
		closeButt.addActionListener(this);

		modeButt = new JToggleButton();
		modeButt.setPreferredSize(new Dimension(22, 22));
		modeButt.setSelectedIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/push.png")));
		modeButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/jump.png")));
		modeButt.addActionListener(this);
		setModusText();

		displayButt = new JToggleButton();
		displayButt.setPreferredSize(new Dimension(22, 22));
		displayButt.setSelectedIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/image.png")));
		displayButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/numbers.png")));
		displayButt.addActionListener(this);
		setDisplayButtonText();

		helpButt = new JButton();
		helpButt.setPreferredSize(new Dimension(22, 22));
		helpButt.setIcon(new ImageIcon(puzzleModel
				.loadImage("img/icons/help.png")));
		helpButt.setToolTipText("Hilfe");
		helpButt.addActionListener(this);

		field = new JTextField(Integer.valueOf(SchiebePuzzleGUI.elements).toString(), 2);
		field.setMaximumSize(new Dimension(22, 22));
		field.addKeyListener(this);

		JLabel fieldLabel = new JLabel("Zeilen:");

		this.add(Box.createHorizontalStrut(5));
		this.add(fieldLabel);
		this.add(field);
		this.add(Box.createHorizontalGlue());
		this.add(newButt);
		this.add(Box.createHorizontalStrut(5));
		this.add(openButt);
		this.add(Box.createHorizontalStrut(5));
		this.add(modeButt);
		this.add(Box.createHorizontalStrut(5));
		this.add(displayButt);
		this.add(Box.createHorizontalStrut(5));
		this.add(sortButt);
		this.add(Box.createHorizontalStrut(5));
		this.add(helpButt);
		this.add(Box.createHorizontalStrut(30));
		this.add(closeButt);
		this.add(Box.createHorizontalStrut(5));
	}
	
	private void setModusText() {
		String modeButtTxt = gui.mode ? springModusTxt : schiebeModusTxt;
		modeButt.setToolTipText(modeButtTxt);
	}

	private void setDisplayButtonText() {
		String displayButtTxt = gui.display ? numbersDisplayTxt : imgDisplayTxt;
		displayButt.setToolTipText(displayButtTxt);
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == newButt) {
			if (gui.display) {
				gui.shuffleElements(gui.imageElements);
				gui.arrangeElements(gui.imageElements);
			} else {
				gui.shuffleElements(gui.numberElements);
				gui.arrangeElements(gui.numberElements);
			}
			mainPanel.revalidate();
		}

		// sorting
		if (e.getSource() == sortButt) {
			mainPanel.removeAll();
			gui.createElements();

			if (gui.display) {
				puzzleModel.setImage(gui.imageElements);
				gui.arrangeElements(gui.imageElements);
			} else {
				gui.arrangeElements(gui.numberElements);
			}
			mainPanel.revalidate();
		}

		// close
		if (e.getSource() == closeButt) {
			System.exit(0);
		}

		// push or jump
		if (e.getSource() == modeButt) {
			gui.mode = !gui.mode;
			setModusText();
		}
		
		// numbers or image
		if (e.getSource() == displayButt) {
			gui.display = !gui.display;
			setDisplayButtonText();
			mainPanel.removeAll();
			puzzleModel.setImage(gui.imageElements);
			if (gui.display) {
				gui.shuffleElements(gui.imageElements);
				gui.arrangeElements(gui.imageElements);
			} else {
				gui.shuffleElements(gui.numberElements);
				gui.arrangeElements(gui.numberElements);
			}
			mainPanel.revalidate();
		}

		// help
		if (e.getSource() == helpButt) {
			new HelpWindow(puzzleModel);
		}

		// open new image
		if (e.getSource() == openButt) {
			new OpenDialog(gui);
		}
	}

	public void keyPressed(KeyEvent e) {
	}

	/**
	 * updating rows/cols by the textfield-entry
	 */
	public void keyReleased(KeyEvent e) {
		if (field.getText().equals(""))
			return;

		for (int i = 0; i < field.getText().length(); i++) {
			if (!Character.isDigit(field.getText().charAt(i))) {
				return;
			}
		}

		int anz = Integer.valueOf(field.getText());
		if (e.getSource() == field && anz >= SchiebePuzzleGUI.minCols && SchiebePuzzleGUI.elements != anz) {
			if (anz > SchiebePuzzleGUI.maxCols) {
				JOptionPane.showMessageDialog(this,
						"Maximale Spalten und Reihenzahl: " + SchiebePuzzleGUI.maxCols);
				return;
			}
			if (anz > 10)
				SchiebePuzzleGUI.fontSize = 15;
			if (anz > 15)
				SchiebePuzzleGUI.fontSize = 12;
			if (anz > 20)
				SchiebePuzzleGUI.fontSize = 10;
			SchiebePuzzleGUI.elements = Integer.valueOf(field.getText()) ;
			gui.rows = SchiebePuzzleGUI.elements;
			gui.cols = SchiebePuzzleGUI.elements;
			gui.rearrange();
		}
	}

	public void keyTyped(KeyEvent e) {
	}

}
